<?php

namespace HCM\Geral\Controllers;

use HCM\Geral\Services\AdmissaoService;
use HCM\Geral\Services\pessoaService;
use HCM\Geral\Services\EsocialService;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\Controller\Crud;
use HCM\Util\DateUtil;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;

class AdmissaoController extends Crud {

    protected $parameterBagColumns = array();

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters,AdmissaoService $admissaoService, pessoaService $pessoaService, ESocialService $eSocialService){
        
        $this->parameterBag = $parameterBag;
        parent::__construct($dataSourceManager);
        $this->hcmParameters = $hcmParameters;
        $this->admissaoService = $admissaoService;
        $this->pessoaService = $pessoaService;
        $this->eSocialService = $eSocialService;
    }
    
    public function buscarPessoaCPF(DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $row = $request->getDataSet()->getRows()[0];
            $row['NRCPFPESSOA'] = str_replace(['.', '-'], ['',''], $row['NRCPFPESSOA']);
            $retorno = $this->admissaoService->buscarPessoaCPF($row['NRCPFPESSOA']);
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('buscarPessoaCPF',$retorno));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function procuraPorDependentes(DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $row = $request->getDataSet()->getRows()[0];
            $retorno = $this->admissaoService->procuraPorDependentes($row['NRPARCNEGOCIO']);
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('procuraPorDependentes',$retorno));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function admissaoVinculo(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $vinculo = null; $parametros = null; $foto = null; $movimentacao = null; $pessoa = null; $endereco = null; $formaComunicacao = null; $contaCorrente = null; $valeTransporte = null; $Dependente = null; $solicitacao = null; $Beneficio = null; $pessoaMesmoCPF = ''; $ignorarVerificacaoQuadroVagas = null;
            foreach ($request->getDataSet()->getRows()[0] as &$row) {
                if(!empty($row['vinculo'])){
                    $vinculo = $row['vinculo'];
                }
                if(!empty($row['cadastroValeTransporte'])){
                    $valeTransporte = $row['cadastroValeTransporte'];
                }
                if(!empty($row['parametros'])){
                    $parametros = $row['parametros'];
                }
                if(!empty($row['movimentacaoCandidato'])){
                    $movimentacao = $row['movimentacaoCandidato'];
                }
                if(!empty($row['Pessoa'])){
                    $pessoa = $row['Pessoa'];
                    $pessoa['NRORG'] = array_key_exists('NRORG', $pessoa) ? $pessoa['NRORG'] : $this->hcmParameters->getNrorgTrabalho(); 
                    $pessoa['IDCOTAPCD'] = array_key_exists('IDCOTAPCD', $pessoa) ? $pessoa['IDCOTAPCD'] : 'N'; 
                }                    
                if(!empty($row['enderecoParc'])){
                    $endereco = $row['enderecoParc'];
                }                    
                if(!empty($row['formaComunicacaoParc'])){
                    $formaComunicacao = $row['formaComunicacaoParc'];
                }                    
                if(!empty($row['contaCorrenteParc'])){
                    $contaCorrente = $row['contaCorrenteParc'];
                }                    
                if(!empty($row['cadastroDependentesCandidato'])){
                    $Dependente = $row['cadastroDependentesCandidato'];
                }
                if(!empty($row['beneficioTitular'])){
                    $Beneficio = $row['beneficioTitular'];
                }
                if(isset($row["ignorarVerificacaoQuadroVagas"])){
                    $ignorarVerificacaoQuadroVagas = $row["ignorarVerificacaoQuadroVagas"];
                }
            }

            if (!$ignorarVerificacaoQuadroVagas) {
                $verificacaoQuadroVagas = $this->getVerificacaoQuadroVagasAdmissao($movimentacao, $parametros);
                
                if ($verificacaoQuadroVagas["hasLimiteQuadroVagas"]) {
                    $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('verificacaoQuadroVagas', $verificacaoQuadroVagas));
                    
                    return;
                }
            }

            if(!empty($request->getDataSet()->getRows()[1]["DSPATH"]) && $request->getDataSet()->getRows()[1]["DSPATH"]!= './assets/images/user.jpg'){
                $foto = $request->getDataSet()->getRows()[1];
            }
            if(!empty($request->getDataSet()->getRows()[2]["NRTIPOSOLICITACAO"])){
                $solicitacao = $request->getDataSet()->getRows()[2];
            }
            $matricula = $this->admissaoService->verificaMatriculaESocial($this->hcmParameters->getNrorgTrabalho(), null, $movimentacao['CDMATRICULAESOCIAL'], $this->hcmParameters->getDtCompetencia(), $movimentacao['NRESTRUTLEGAL'], null);
            if($pessoa){
                $pessoaMesmoCPF = $this->pessoaService->validaCpfExistePessoa($pessoa, false);
            }
            
            if($pessoaMesmoCPF){
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($pessoaMesmoCPF, DTO\Response\Message::TYPE_WARNING));
                return;
            }
                
            if($matricula['type'] == 'SUCESS'){
                $retorno = $this->admissaoService->admissaoVinculo($vinculo, $parametros, $foto, $movimentacao, $pessoa, $endereco, $formaComunicacao, $contaCorrente, $Dependente, $solicitacao, $Beneficio, $valeTransporte);
                
                if(!empty($retorno) && $retorno['IDAPROVADMISSAO'] == 'N' && array_key_exists('IDENVIAESOCIAL', $vinculo) && $vinculo['IDENVIAESOCIAL'] == 'S'){
                    $vinculo['NRCHAVE'] = $retorno['NRVINCULOM'];
                    $vinculo['NRVINCULOM'] = $retorno['NRVINCULOM'];
                    $this->eSocialService->envioEsocialDescentralizado($vinculo, $response);
                }
                
                $response->addNotification(new \Zeedhi\Framework\DTO\Response\Notification("Admissão realizada com sucesso", DTO\Response\Notification::TYPE_SUCCESS));
            }else{
                
                $mensagem = '';
                if($pessoaMesmoCPF){
                    $mensagem = '- '.$matricula['message'].'<br>- '.$pessoaMesmoCPF;
                }else{
                    $mensagem = $matricula['message']; 
                }
                
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($mensagem, DTO\Response\Message::TYPE_ERROR));
            }
        } catch (\Exception $e) {
            if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }else{      
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
            }  
        }
    }

    private function getVerificacaoQuadroVagasAdmissao($movimentacao, $parametros): array
    {
        $movimentacoes = [
            'GERENCIAL' => [
                'TIPOESTRUTURA' => $movimentacao['NRTIPOESTRUTURA_GEREN'],
                'NRESTRUTURA' => $movimentacao['NRESTRUTGEREN'],
                'NMESTRUTURA' => $movimentacao['NMRAZSOCESTRUT_GEREN'],
                'NROCUPACAO' => $parametros['NROCUPACAOM'],
                'NMOCUPACAO' => $parametros['NMOCUPACAOH']
            ],
            'LEGAL' => [
                'TIPOESTRUTURA' => $movimentacao['NRTIPOESTRUTURA_LEGAL'],
                'NRESTRUTURA' => $movimentacao['NRESTRUTLEGAL'],
                'NMESTRUTURA' => $movimentacao['NMRAZSOCESTRUT_LEGAL'],
                'NROCUPACAO' => $parametros['NROCUPACAOM'],
                'NMOCUPACAO' => $parametros['NMOCUPACAOH']
            ],
            'SINDICAL' => [
                'TIPOESTRUTURA' => $movimentacao['NRTIPOESTRUTURA_SINDICAL'] ?? null,
                'NRESTRUTURA' => $movimentacao['NRESTRUTSIND'] ?? null,
                'NMESTRUTURA' => $movimentacao['NMRAZSOCESTRUT_SINDICAL'] ?? null,
                'NROCUPACAO' => $parametros['NROCUPACAOM'],
                'NMOCUPACAO' => $parametros['NMOCUPACAOH']
            ],
            'ADICIONAL' => [
                'TIPOESTRUTURA' => $movimentacao['NRTIPOESTRUTURA_ADICIONAL'] ?? null,
                'NRESTRUTURA' => $movimentacao['NRESTRUTADICIONAL'] ?? null,
                'NMESTRUTURA' => $movimentacao['NMRAZSOCESTRUT_ADICIONAL'] ?? null,
                'NROCUPACAO' => $parametros['NROCUPACAOM'],
                'NMOCUPACAO' => $parametros['NMOCUPACAOH']
            ]
        ];
    
        return $this->admissaoService->getVerificacaoQuadroVagas($movimentacoes);
    }

    public function admissaoPreliminarSimplificada(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            foreach ($request->getDataSet()->getRows() as $row) {
                $nrcpfpessoa = $row['NRCPFPESSOA'];
                $nrpessoa = isset($row['NRPESSOA']) ? $row['NRPESSOA'] : null;
                $nmpessoa = $row['NMPESSOA'];
                $dtnascpessoa = $row['DTNASCPESSOA'];
                $idmultvinc = $row['IDMULTVINC'];
                $nrtipovinculom = $row['NRTIPOVINCULOM'];
                $dtadmissaovinc = $row['DTADMISSAOVINC'];
                $cdmatricula = isset($row['CDMATRICULA']) ? $row['CDMATRICULA'] : null;
                $cdmatriculaesocial = isset($row['CDMATRICULAESOCIAL']) ? $row['CDMATRICULAESOCIAL'] : '';
                $nrvinculoempreg = $row['NRVINCULOEMPREG'];
                $dtfimcontrdetermin = isset($row['DTFIMCONTRDETERMIN']) ? $row['DTFIMCONTRDETERMIN'] : null;
                $nrocupacaom = $row['NROCUPACAOM'];
                $cdtiposalario = $row['CDTIPOSALARIO'];
                $vrsalario = $row['VRSALARIO'];
                $nrtipoestrutura_legal = $row['NRTIPOESTRUTURA_LEGAL'];
                $nrestrutlegal = $row['NRESTRUTLEGAL'];
                $nrtipoestrutura_geren = $row['NRTIPOESTRUTURA_GEREN'];
                $nrestrutgeren = $row['NRESTRUTGEREN'];
                $nrtipoestrutura_sindical = isset($row['NRTIPOESTRUTURA_SINDICAL']) ? $row['NRTIPOESTRUTURA_SINDICAL'] : null;
                $nrestrutsind = isset($row['NRESTRUTSIND']) ? $row['NRESTRUTSIND'] : null;
                
                if($nrcpfpessoa){
                    $pessoa = array(
                        'NRORG' => $this->hcmParameters->getNrorgTrabalho(),
                        'NRPESSOA' => $nrpessoa,
                        'NRCPFPESSOA' => $nrcpfpessoa,
                    );
                    $pessoaMesmoCPF = $this->pessoaService->validaCpfExistePessoa($pessoa, false);
                }
                
                if($pessoaMesmoCPF){
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($pessoaMesmoCPF, DTO\Response\Message::TYPE_WARNING));
                }
                
                $retorno = $this->admissaoService->admissaoPreliminarSimplificada(
                    $nrcpfpessoa, $nrpessoa, $nmpessoa, $dtnascpessoa, $idmultvinc, $nrtipovinculom, $dtadmissaovinc, $cdmatricula, $cdmatriculaesocial, $nrvinculoempreg, $dtfimcontrdetermin, $nrocupacaom, $cdtiposalario, $vrsalario,
                    $nrtipoestrutura_legal, $nrestrutlegal, $nrtipoestrutura_geren, $nrestrutgeren, $nrtipoestrutura_sindical, $nrestrutsind
                );
                
                $response->addNotification(new \Zeedhi\Framework\DTO\Response\Notification("Vínculo $retorno admitido com sucesso!", DTO\Response\Notification::TYPE_SUCCESS));
            }
        } catch (\Exception $e) {
            if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }else{      
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
            }  
        }
    }
    
    public function admissaoColetivaVinc(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $action = $request->getRoutePath();
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();

            foreach ($request->getDataSet()->getRows() as &$row) {
                $nrestrutlegal = (isset($row['NRESTRUTLEGAL']) && !empty($row['NRESTRUTLEGAL']))?$row['NRESTRUTLEGAL']:null;
                $nrestrutgerenlist = (isset($row['NRESTRUTGEREN']) && !empty($row['NRESTRUTGEREN']))?$row['NRESTRUTGEREN']:'A';
                $nrvinculomlist = (isset($row['NRVINCULOM']) && !empty($row['NRVINCULOM']))?$row['NRVINCULOM']:'A';
                $dtrescisao_ini = $row['DTRESCISAOVINC_INICIO'];
                $dtrescisao_fim = $row['DTRESCISAOVINC_FIM'];
                $dtadmissao = $row['DTADMISSAOVINC'];
                
                if ($action == '/admissaoColetivaVinc') {
                    $status = $this->admissaoService->admissaoColetiva($nrorg, $nrestrutlegal, $nrestrutgerenlist, $nrvinculomlist, $dtrescisao_ini, $dtrescisao_fim, $dtadmissao);
                }
            }
            $response->addNotification(new DTO\Response\Notification('',$status));
        } catch (\Exception $e) {
            if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }else{
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
            }  
        }
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
    
    public function getHorasSemanais(DTO\Request\DataSet $request, DTO\Response $response) {
        
        foreach ($request->getDataSet()->getRows() as &$row) {
            $row['NRORG'] = $this->hcmParameters->getNrorgTrabalho();
            $row['DTMESCOMPETENC'] = DateUtil::getUltimoDiaDoMes($this->hcmParameters->getDtCompetencia(),DateUtil::FORMATO_BRASILEIRO,true)->format('d/m/Y');
            $result = $this->admissaoService->getHorasSemanais($row);
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('quantidadeHoras', array($result)));
        }
    }
}